﻿using System;
using System.Collections.Generic;
using System.CommandLine;
using System.CommandLine.Invocation;
using System.Linq;
using System.Reflection;
using UpToolLib;
using UpToolLib.DataStructures;
using UpToolLib.v1.Tool;
using UpToolLib.v2;

namespace UpToolCLI
{
    public static class CacheManagement
    {
        public static void RegisterCommands(RootCommand rootCommand)
        {
            rootCommand.AddCommand(new Command("list", "Lists installed packages")
            {
                Handler = CommandHandler.Create(List)
            });

            Command search = new("search", "Search for packages")
            {
                new Argument<string>("identifier", "Something to identify the app")
            };
            search.Handler = CommandHandler.Create<string>(Search);
            rootCommand.AddCommand(search);

            Command show = new("show", "Shows package info")
            {
                new Argument<string>("identifier", "Something to identify the app")
            };
            show.Handler = CommandHandler.Create<string>(Show);
            rootCommand.AddCommand(show);

            rootCommand.AddCommand(new Command("update", "Updates the cache")
            {
                Handler = CommandHandler.Create(Update)
            });
        }

        private static void List()
        {
            Program.Lib.V2.RepoManagement.GetReposFromDisk();
            Console.WriteLine(Program.Lib.V1.Apps.Where(s => s.Value.Status.Contains(Status.Installed))
                .ToStringTable(new[]
                    {
                        "Name", "State", "Guid"
                    },
                    u => u.Value.Name,
                    u => u.Value.Local ? "Local" :
                        u.Value.Status.Contains(Status.Updatable) ? "Updatable" : "None",
                    u => u.Key));
        }

        private static void Search(string identifier)
        {
            Program.Lib.V2.RepoManagement.GetReposFromDisk();
            App[] apps = Program.Lib.V1.AppExtras.FindApps(identifier);
            Console.WriteLine($"Found {apps.Length} app(s)");
            if (apps.Length > 0)
                Console.WriteLine(apps.ToStringTable(new[]
                    {
                        "Name", "Guid"
                    },
                    u => u.Name,
                    u => u.Id));
        }

        private static void Show(string identifier)
        {
            Program.Lib.V2.RepoManagement.GetReposFromDisk();
            App[] apps = Program.Lib.V1.AppExtras.FindApps(identifier);
            if (apps.Length == 0)
                Console.WriteLine("Package not found.");
            else
                Console.WriteLine(apps.First());
        }

        private static void Update()
        {
            Console.WriteLine("Fetching Repos...");
            Program.Lib.V2.RepoManagement.FetchRepos();
            Program.Lib.V2.RepoManagement.GetReposFromDisk();
            Console.WriteLine();
            IEnumerable<App> tmp = Program.Lib.V1.Apps.Where(s =>
                s.Value.Status.Contains(Status.Updatable)).Select(s => s.Value);
            IEnumerable<App> apps = tmp as App[] ?? tmp.ToArray();
            int updatableCount = apps.Count();
            Console.WriteLine(updatableCount == 0
                ? "All up-to-date"
                : $@"Found {updatableCount} Updates:
{string.Join(Environment.NewLine, apps.Select(s => $"- {s.Name} ({s.Version})"))}");
            Version vLocal = Assembly.GetExecutingAssembly().GetName().Version;
            Version vOnline = Program.Lib.V2.UpdateChecker.Check().OnlineVersion;
            if (vLocal < vOnline)
                Console.WriteLine($"uptool is outdated ({vLocal} vs {vOnline}), update using \"uptool upgrade-self\"");
        }
    }
}