﻿using System;
using System.CommandLine;
using System.CommandLine.Invocation;
using System.Diagnostics;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Reflection;
using System.Security.Cryptography;
using UpToolLib;
using UpToolLib.DataStructures;
using UpToolLib.v1.Tool;
using UpToolLib.v2;
using Process = System.Diagnostics.Process;

namespace UpToolCLI
{
    public class Other
    {
        public static void RegisterCommands(RootCommand rootCommand)
        {
            Command command = new("upgrade-self", "Upgrades UpToolCLI")
            {
                new Option<bool>(new[] {"--force", "-f"}, "Overwrites older files")
            };
            command.Handler = CommandHandler.Create<bool>(UpgradeSelf);
            rootCommand.AddCommand(command);

            Command start = new("start", "Starts an app")
            {
                new Argument<string>("identifier", "Something to identify the app"),
                new Option<string>(new[] {"--arguments", "--args", "-a"}, () => "", "The arguments to run the file with"),
                new Option<string>(new[] {"--file", "-f"}, () => null, "The file to run instead of the MainFile"),
                new Option<bool>(new[] {"--waitForExit", "-wait"}, "Waits until the program quits")
            };
            start.Handler = CommandHandler.Create<string, string, string, bool>(Start);
            rootCommand.AddCommand(start);
        }

        public static void UpgradeSelf(bool force)
        {
            PathTool pathTool = Program.Lib.V1.PathTool;
            UpdateCheck updateCheck = Program.Lib.V2.UpdateChecker.Check();
            if (!force && Assembly.GetExecutingAssembly().GetName().Version >= updateCheck.OnlineVersion)
                Console.WriteLine("Already up-to-date");
            else
            {
                Console.WriteLine("Downloading latest");
                (bool success, byte[] dl) = Program.Functions.Download(updateCheck.Installer);
                if (!success)
                    throw new Exception("Failed to update");
                Console.WriteLine("Verifying");
                using (SHA256CryptoServiceProvider sha256 = new())
                {
                    string pkgHash = BitConverter.ToString(sha256.ComputeHash(dl)).Replace("-", string.Empty).ToUpper();
                    if (pkgHash != updateCheck.InstallerHash)
                        throw new Exception($@"The hash is not equal to the one stored in the repo:
Package: {pkgHash}
Online: {updateCheck.InstallerHash}");
                }
                Console.WriteLine("Installing");
                if (Directory.Exists(pathTool.GetRelative("Install", "tmp")))
                    Directory.Delete(pathTool.GetRelative("Install", "tmp"), true);
                Directory.CreateDirectory(pathTool.GetRelative("Install", "tmp"));
                using (MemoryStream ms = new(dl))
                {
                    using ZipArchive ar = new(ms);
                    ar.ExtractToDirectory(pathTool.GetRelative("Install", "tmp"), true);
                }
                string file = pathTool.GetRelative("Install", "tmp", "Installer.exe");
                Console.WriteLine($"Starting {file}");
                Process.Start(new ProcessStartInfo
                {
                    FileName = file,
                    Arguments = "i",
                    WorkingDirectory = pathTool.GetRelative("Install"),
                    UseShellExecute = false
                });
            }
        }

        private static void Start(string identifier, string arguments, string file, bool waitForExit)
        {
            Program.Lib.V2.RepoManagement.GetReposFromDisk();
            App[] apps = Program.Lib.V1.AppExtras.FindApps(identifier);
            if (apps.Length == 0)
                Console.WriteLine("Package not found.");
            else
            {
                App tmp = apps.First();
                if (tmp.Runnable)
                {
                    Console.WriteLine($"Starting {tmp.Name}");
                    Process tmp1 = Program.Lib.V1.AppExtras.RunApp(tmp, file ?? tmp.MainFile, arguments);
                    if (waitForExit)
                        tmp1.WaitForExit();
                }
                else
                    Console.WriteLine($"{tmp.Name} is not runnable");
            }
            Console.WriteLine("Done!");
        }
    }
}