using System;
using System.Collections.Generic;
using UpToolLib.DataStructures;
using UpToolLib.v1.Tool;

namespace UpToolLib.v2.TaskQueue
{
    public class TaskFactory
    {
        private IExternalFunctionality _platform;
        private readonly AppInstall _installer;
        private readonly AppExtras _extras;
        private readonly PathTool _pathTool;
        private readonly AppFactory _appFactory;
        private readonly IDictionary<Guid, App> _apps;

        internal TaskFactory(IExternalFunctionality platform, AppInstall installer, AppExtras extras, PathTool pathTool, AppFactory appFactory, IDictionary<Guid, App> apps)
        {
            _platform = platform;
            _installer = installer;
            _extras = extras;
            _pathTool = pathTool;
            _appFactory = appFactory;
            _apps = apps;
        }

        public KnownAppTask CreateInstall(App app, Action? postInstall = null) => new InstallTask(_platform, _installer, app, postInstall);

        public KnownAppTask CreateRemove(App app, Action? postInstall = null) => new RemoveTask(_platform, _extras, app, postInstall);

        public KnownAppTask CreateUpdate(App app, Action? postInstall = null) => new UpdateTask(_platform, _extras, app, postInstall);

        public AppTask CreateUpload(string zipFile, string name, Action? postInstall = null) => new UploadTask(_platform, _installer, _appFactory, _pathTool, _apps, zipFile, name, postInstall);
    }
}