﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using UpToolLib.DataStructures;
using UpToolLib.v1;
using UpToolLib.v1.Tool;

namespace UpToolLib.v2.TaskQueue
{
    public class UploadTask : AppTask
    {
        private readonly string _name;
        private readonly Action? _postInstall;
        private readonly AppInstall _installer;
        private readonly AppFactory _appFactory;
        private readonly PathTool _pathTool;
        private readonly IDictionary<Guid, App> _apps;
        public readonly string ZipFile;

        internal UploadTask(IExternalFunctionality platform, AppInstall installer, AppFactory appFactory, PathTool pathTool, IDictionary<Guid, App> apps, string zipFile, string name, Action? postInstall) : base(platform)
        {
            _installer = installer;
            _appFactory = appFactory;
            _pathTool = pathTool;
            _apps = apps;
            ZipFile = zipFile;
            _name = name;
            _postInstall = postInstall;
        }

        public override void Run()
        {
#if !DEBUG
            try
            {
#endif
                Guid id = Guid.NewGuid();
                while (_apps.ContainsKey(id) || Directory.Exists(_pathTool.GetAppPath(id)))
                    id = Guid.NewGuid();
                App appI = _appFactory.CreateApp(_name, "Locally installed package, removal only",
                    UpToolLibV1.MinimumVer, "", true, "", id, Color.Red, Platform.GetDefaultIcon(), false, "");
                _installer.InstallZip(ZipFile, appI, true);
                _postInstall?.Invoke();
#if !DEBUG
            }
            catch (Exception e1)
            {
                Platform.OkDialog("Upload failed.\r\nException=" + e1);
            }
#endif
        }

        public override string ToString() => $"Install local {Path.GetFileName(ZipFile)}";
    }
}